// LoginServlet.java  for CISC474, H07, P. Conrad, Spring 2007

package edu.udel.cisc474.pconrad.BeerAdvisor.controller;

import edu.udel.cisc474.pconrad.BeerAdvisor.model.BeerExpert;
import edu.udel.cisc474.pconrad.BeerAdvisor.model.LoginChecker;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*; // for List and Iterator


/**
 * <p>
 * LoginServlet is the controller for the login features of an MVC based webapp
 * called BeerAdvisor. The LoginServlet takes input from a login page,
 * and passes it to a model class and a view for processing.
 * 
 * @author    P.Conrad
 */

public class LoginServlet extends HttpServlet {

    /** 
     * doPost is the event handler for the HTTP post method.
     */

    public void doPost(HttpServletRequest request,
		       HttpServletResponse response)
		       throws IOException, ServletException {


	// if there was already a session associated with this client,
	// get that session.  The parameter false means "don't create a new
	// one if there is one already there".
	HttpSession session = request.getSession(false);

	if (session!=null){
	    // That means there must have already been a session.
	    // So invalidate that one.  We're going to establish a new one.
	    session.invalidate();
	}

	session = request.getSession(); // now we know we have a new one!

	String userid = request.getParameter("userid");
	String password = request.getParameter("password");

	// get the database parameters from the deployment descriptor
	// (i.e. the web.xml file.)  These are set as webapp level parameters,
	// i.e. servlet context parameters. (see HFSJ, p. 156, 157 and
	// thereabouts...)

	String jdbcURL = getServletContext().getInitParameter("jdbcURL");
	String mySQLuser = getServletContext().getInitParameter("mySQLuser");
	String mySQLpw = getServletContext().getInitParameter("mySQLpw");

	// create a loginChecker object to do the database work for us

	LoginChecker lc = new LoginChecker(jdbcURL,mySQLuser,mySQLpw);


	// Check whether the login was valid using the userIsValid
	// method of the LoginChecker object.  Then set a request attribute
	// that the JSP file can check
	    
	Boolean loginWasValid = lc.userIsValid(userid,password);
	    
	if (loginWasValid)
	    {


		
		// beerSelect.jsp will get the attribute from the session

		session.setAttribute("loginWasValid",loginWasValid);

		// Store some useful attributes in the session so that
		// we can greet the user by name.

		// The following code is very inefficient--we are connecting
		// to the SQL database three times and running three queries
		// when only one would do the job.  Refactoring this to 
		// improve the efficiency is left as an exercise to the
		// CISC474 student!
		
		String fname = lc.getField("fname",userid,password);
		String lname = lc.getField("lname",userid,password);
		   
		session.setAttribute("fname",fname);
		session.setAttribute("lname",lname);

		// Now, off to select a beer, where we can greet the user
		// by name!

		RequestDispatcher view =
		    request.getRequestDispatcher("beerSelect.jsp");
		
		view.forward(request, response);
	    }
	else
	    {
		// index.jsp will get the attribute from the request

		request.setAttribute("loginWasValid",loginWasValid);

		RequestDispatcher view =
		    request.getRequestDispatcher("index.jsp");
		
		view.forward(request, response);
	    }

    }

    
}

		       
