// LoginChecker.java  for CISC474, H07, P. Conrad, Spring 2007

package edu.udel.cisc474.pconrad.BeerAdvisor.model;
import java.util.*;
import java.sql.*;   // Needed for doing database stuff

/**
 * <p>
 * LoginChecker checks whether a given login/password is valid.
 * This class might be extended later with methods that do things such as
 * look up the login and password associated with a particular email
 * address, so that this information can be emailed to that user.
 *
 * This class provides an abstraction of the actual data
 * source, which can be hard coded values, or an SQL database.
 * <p>
 * LoginChecker represents the "Model" part of a login 
 * system based on the
 * Model-View-Controller design pattern.
 * <p>
 * 
 * @author    P.Conrad
 */


public class LoginChecker {

    /**
     * Create a LoginChecker object that gets its information from a MySQL
     * relational database, specified by the paramaters passed in to the constructor.
     *
     * @param jdbcURL  for example,
     *                 <code>"jdbc:mysql://cisc474.acad.cis.udel.edu/pconrad"
     *                 The <code>"jdbc.mysql://"</code> should be
     *                 exactly that.  The
     *                 <code>cisc474.acad.cis.udel.edu</code> part is
     *                 the host where the database is running, and the
     *                 <code>pconrad</code> part is the name of the
     *                 <em>database</em>, not the userid.  In this
     *                 case, those happen to be the same, but don't be
     *                 confused on this point.
     * @param mySQLuser for example, <code>"pconrad"</code>---the MySQL
     *                 username, <em>not</em> your Unix username!
     * @param mySQLpw  for example, <code>"mySecret123"</code>---the
     *                 MySQL password, <em>not</em> your Unix
     *                 password!
     */

    private String jdbcURL_;
    private String mySQLuser_;
    private String mySQLpw_;
    private final String prefix_ = "BA_";

     /* Explanation of prefix_: Many hosting providers only allow a
        user one database.  So, if you want to have multiple
        applications running, it is helpful to prefix all of your
        table names with some prefix, e.g. "BA_" to distinguish them
        from tables for some other application.</p> <p>This is
        admiteddly a hack.  In a perfect world, you'd separate
        applications into different databases, but in the real world,
        sometimes this is not feasible. */

    public LoginChecker(String jdbcURL, String mySQLuser, String mySQLpw)
    {

	// initialize the private data members

	System.out.println("In LoginChecker:" + 
			   " jdbcURL=" + jdbcURL +
			   " mySQLuser=" + mySQLuser + 
			   " mySQLpw=" + mySQLpw);

	jdbcURL_ = jdbcURL;
	mySQLuser_ = mySQLuser;
	mySQLpw_ = mySQLpw;

    }


/**
 * Given a color of beer, returns a list of beers that
 * match that color. 
 * 
 * @param userid       the username to search for
 * @param password     the password to search for
 * @return             true if username,password was in the database, false otherwise
 * @see LoginCheckerTest
 */

    public Boolean userIsValid(String userid, String password) {

	// Create the connection to the db	
	
	Connection myCon;
	Statement myStmt;


	// Make sure we can load the MySQL driver 

	try {
	    Class.forName("com.mysql.jdbc.Driver").newInstance();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

	System.out.println("In LoginChecker.userIsValid:" + 
			   " jdbcURL_=" + jdbcURL_ +
			   " mySQLuser_=" + mySQLuser_ + 
			   " mySQLpw_=" + mySQLpw_);


	try {
	    myCon = DriverManager.getConnection(jdbcURL_,mySQLuser_,mySQLpw_);
	}
	catch (SQLException e) {
	    e.printStackTrace();
	    return false;
	}

	// Create a statement for SQL queries

	try {
	    myStmt = myCon.createStatement();
	}
	catch (Exception e){
	    e.printStackTrace();
	    return false;
	}
	
	// Set up the query.  Use a ? where we will put in user input, and use
	// the "prepared statement" technique, in order to avoid SQL Injection attacks

	String tableName = prefix_ + "Users";
	String query = "SELECT userid, password FROM " + tableName + "  WHERE userid=? AND password=?";
	
	// Turn our query into a prepared statement, and then stick the value of "color"
	// in where the first ? appears.   The setString method of prepared statement 
	
	ResultSet rs;
	
	try {
	    PreparedStatement prepStmt = myCon.prepareStatement(query);
	    prepStmt.setString(1, userid);
	    prepStmt.setString(2, password);
	    rs = prepStmt.executeQuery();
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return false;
	}

	// ** change next line to true to make JUnit tests fail
	final boolean stubForCheckingJUnitTests = false;

	if (stubForCheckingJUnitTests)
	    return false; // or could be true---whatever you need to check

	try {
	    if (rs.next()) 
		return true; // we got at least one row
	    else
		return false; // there was no row
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return false;
	}	

    }

    /**
     *  A method to look up  a field in the user database
     *
     */

    // This is horrible cut and paste code.   This is begging for
    // refactoring.  Someone who does a nice job of refactoring it and
    // posts their refactored code on a web site, along with a wiki page
    // linking to an explanation of what they did... that person could
    // earn some nice choice points!
    //
    // Perhaps the way to do it is to abstract a "ValidatedUser" into an
    // object, and store that Object (along with all its attributes)
    // in the session?  That way, we don't have to keep going back to
    // the database...  we can just have methods such as getFname and getLname

    public String getField(String field, String userid, String password) {

	// Create the connection to the db	
	
	Connection myCon;
	Statement myStmt;


	// Make sure we can load the MySQL driver 

	try {
	    Class.forName("com.mysql.jdbc.Driver").newInstance();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

	System.out.println("In LoginChecker.userIsValid:" + 
			   " jdbcURL_=" + jdbcURL_ +
			   " mySQLuser_=" + mySQLuser_ + 
			   " mySQLpw_=" + mySQLpw_);


	try {
	    myCon = DriverManager.getConnection(jdbcURL_,mySQLuser_,mySQLpw_);
	}
	catch (SQLException e) {
	    e.printStackTrace();
	    return "";
	}

	// Create a statement for SQL queries

	try {
	    myStmt = myCon.createStatement();
	}
	catch (Exception e){
	    e.printStackTrace();
	    return "";
	}
	
	// Set up the query.  Use a ? where we will put in user input, and use
	// the "prepared statement" technique, in order to avoid SQL Injection attacks

	String tableName = prefix_ + "Users";
	String query = "SELECT " + field + " FROM " + tableName + "  WHERE userid=? AND password=?";
	
	// Turn our query into a prepared statement, and then stick the value of "color"
	// in where the first ? appears.   The setString method of prepared statement 
	
	ResultSet rs;
	
	try {
	    PreparedStatement prepStmt = myCon.prepareStatement(query);
	    prepStmt.setString(1, userid);
	    prepStmt.setString(2, password);
	    rs = prepStmt.executeQuery();
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return "";
	}

	// ** change next line to true to make JUnit tests fail
	final boolean stubForCheckingJUnitTests = false;

	if (stubForCheckingJUnitTests)
	    return ""; 

	try {
	    if (rs.next()) 
		return rs.getString(field); // we got at least one row
	    else
		return ""; // there was no row
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return "";
	}	

    }


    /**
       main is a command line "view/controller" that 
       takes a userid and password as first two command line args, 
       calls userIsValid, and prints the result.
       
    */

    public static void main(String args[]) {

	// check command line arguments

	if (args.length != 5)
	    {
		System.err.println("Usage: java LoginChecker userid password jdbcURL mySQLuser mySQLpw");
		System.exit(1);
	    }

	String userid = args[0];
	String password = args[1];
	String jdbcURL = args[2];
	String mySQLuser = args[3];
	String mySQLpw = args[4];

	LoginChecker lc = new LoginChecker(jdbcURL,mySQLuser,mySQLpw);
	
	Boolean result = lc.userIsValid(userid,password);

	if (result) {
	    System.out.println("login was valid");
	    System.exit(0);
	}
	else {
	    System.out.println("login was not valid");
	    System.exit(1);
	}
	
    } // main


}
